/**
 *  @file bltwebd.c
 *
 *  @author Collin R. Mulliner <collin@mulliner.org>
 *
 *  This is part of the Bluetooth Location Tracker (BLT) project
 */

#include "bltwebd.h"
#include <getopt.h>

struct blt_loc_names_t locations;
struct blt_data_client_session_t client;
struct blt_dev_names_t devNames;
int running;

int main(int argc, char **argv)
{
	const char *DEFAULT_outputFile = {"blt_input.php\0"};
	const char *DEFAULT_server = {"localhost\0"};
	int c;
	char *p;
	struct readbuf_t buf;	/**< read buffer */
	char buffer[1024];
	int dumpDelay = 0;
	int rb;
	unsigned int dumpAge = 0;
	struct blt_device_status_t currentDev;
	char *outputFile = NULL;
		
	
	/* basic init */
	client.status = BLT_DC_DISCONNECTED;
	client.data_ip = NULL;
	
	/* get arguments */
	while ((c = getopt(argc, argv, "?s:d:a:o:")) != -1) {
		switch (c) {
			case 's':
				client.data_ip = (char*) malloc(strlen(optarg)+1);
				strcpyendchr(optarg, client.data_ip, ':', strlen(optarg));
				p = strchr(optarg, ':');
				if (p > 0) client.data_port = atoi(p+1);
				break;
			case 'd':
				dumpDelay = atoi(optarg);
				if (dumpDelay == 0) dumpDelay = 60;
				break;
			case 'a':
				dumpAge = atoi(optarg);
				break;
			case 'o':
				outputFile = optarg;
				break;
			case '?':
				fprintf(stderr, "syntax: %s -s<server:port> -d<dumpDelay> -o<outputFile> -a<dumpAge>\n", argv[0]);
				return(0);
				break;
			default:
				break;
		}
	}

	/* defaults */
	if (outputFile == NULL) outputFile = DEFAULT_outputFile;
	#ifdef DEBUG
	fprintf(stderr, "writing to: %s\n", outputFile);
	#endif
	if (dumpDelay == 0) dumpDelay = 60;
	#ifdef DEBUG
	fprintf(stderr, "dumping data every  %d seconds\n", dumpDelay);
	#endif
	if (client.data_ip == NULL) client.data_ip = DEFAULT_server;
	if (client.data_port == 0) client.data_port = BLT_SERVER_DEFAULT_PORT;
	#ifdef DEBUG
	fprintf(stderr, "using blt server at: %s:%d\n",client.data_ip, client.data_port);
	#endif
	
	if (dumpAge == 0) dumpAge = (60*30);

	/* load the location names */
	blt_ln_init(&locations);
	blt_ln_load(&locations);

	/* init device status store */
	blt_dn_init(&devNames);
	
	/* init readbuf */
	readbufInit(&buf, 4096);

	running = 1;
	do {
		/* re-connect loop */
		while (client.status == BLT_DC_DISCONNECTED && running == 1) {
			#ifdef DEBUG
			fprintf(stderr, "connecting server...\n");
			#endif
			if (blt_client_connect(&client)) {
				sprintf(buffer, "data(list_all_devices)\r\n");
				if (nwrite(client.mySocket, buffer, strlen(buffer)) == -1) {
					client.status = BLT_DC_DISCONNECTED;
				}
		
				sprintf(buffer, "data(push_all_devices)\r\n");
				if (nwrite(client.mySocket, buffer, strlen(buffer)) == -1) {
					client.status = BLT_DC_DISCONNECTED;
				}
				else {
					#ifdef DEBUG
					fprintf(stderr, "connected\n");
					#endif
				}
			}
			else {
				#ifdef DEBUG
				fprintf(stderr, "connection failed/lost ... sleeping\n");
				#endif
				sleep(5);
			}
		}
		
		if ((rb = readbuf(&buf, client.mySocket, buffer, 1024, dumpDelay)) == 1) {
			if (strstr(buffer, "data(") == NULL) {
				if (blt_parseDataLine(buffer, &currentDev)) {
					blt_dn_write(&devNames, &currentDev);
				}
			}
		}
		else if (rb == 0) {
			#ifdef DEBUG2
			fprintf(stderr, "dumping!\n");
			#endif
			blt_dumpAll(&devNames, &locations, outputFile, dumpAge);
		}
		else {
			client.status = BLT_DC_DISCONNECTED;
		}
	} while(running);

	return(1);
}
