/**
 *  @file location.c
 *
 *  @author Collin R. Mulliner <collin@mulliner.org>
 *
 *  This is part of the Bluetooth Location Tracker (BLT) project
 */

#include "bltwebd.h"

int blt_ln_load(struct blt_loc_names_t *ln)
{
	FILE *fp;
	char buffer[1024];
	struct blt_location_name_t loc;
	struct blt_location_name_t *tmp;
	char *p;
	unsigned int maxLocs = 0;
	
	
	if ((fp = fopen(BLT_LOCATION_NAMES_FILE, "r")) == NULL) {
		return(0);
	}
	
	while (!feof(fp)) {
		memset(buffer, 0, 1024);
		memset(&loc, 0, sizeof(struct blt_location_name_t));
		fgets(buffer, 1024, fp);
		sscanf(buffer, "%d %d", &loc.la, &loc.lg);
		p = strstr(buffer, "\"");
		if (p > 0) strcpyendchr(p+1, loc.name, '"', BLT_LOCATION_NAME_LENGTH+1);
		else continue;
		
		#ifdef DEBUG
		fprintf(stderr, "%d %d \"%s\"\n", loc.la, loc.lg, loc.name);
		#endif
		
		if (ln->numLocations+1 > maxLocs) {
			tmp = ln->locs;
			ln->locs = malloc((ln->numLocations+10) * sizeof(struct blt_location_name_t));
			if (tmp != NULL) memcpy(ln->locs, tmp, ln->numLocations * sizeof(struct blt_location_name_t));
			maxLocs += 10;
		}
		memcpy(&ln->locs[ln->numLocations], &loc, sizeof(struct blt_location_name_t));
		ln->numLocations++;
		
		if (feof(fp)) break;
	}
	
	fclose(fp);
	
	return(1);
}

void blt_ln_destroy(struct blt_loc_names_t *ln)
{
	if (ln->locs != NULL) {
		free(ln->locs);
		ln->locs = NULL;
		ln->numLocations = 0;
	}
}

void blt_ln_init(struct blt_loc_names_t *ln)
{
	ln->locs = NULL;
	ln->numLocations = 0;
}
