/**
 *  @file network.c
 *
 *  @author Collin R. Mulliner <collin@mulliner.org>
 *
 *  This is part of the Bluetooth Location Tracker (BLT) project
 */

#include "bltwebd.h"

/**
 *  @brief connect to server
 *
 *  @param self session to start
 *
 *  @returns 1 on success, 0 on failure
 */
int blt_client_connect(struct blt_data_client_session_t *self)
{
	struct hostent *hostinfo;
	struct sockaddr_in sin;


	sin.sin_port = htons((u_short) self->data_port);
	
	if ((hostinfo = gethostbyname(self->data_ip))) {
		bcopy(hostinfo->h_addr, (char *) &sin.sin_addr, hostinfo->h_length);
		sin.sin_family = AF_INET;
	}
	else {
		#ifdef DEBUG
		fprintf(stderr, "can't gethostbyname for: %s:%d\n", self->data_ip, self->data_port);
		#endif
		return(0);
	}

	if ((self->mySocket = socket(AF_INET, SOCK_STREAM, 0))) {}
	else {
		#ifdef DEBUG
		fprintf(stderr, "can't create socket\n");
		#endif
		return(0);
	}

	if (connect(self->mySocket, (struct sockaddr*) &sin, sizeof(sin)) < 0) {
		#ifdef DEBUG
		fprintf(stderr, "can't connect to server\n");
		#endif
		return(0);
	}
	
	self->status = BLT_DC_CONNECTED;
		
	return(1);
}

/**
 *  @brief disconnect from server
 *
 *  @param self the blt session to end
 *
 *  @returns 1 on success, 0 on failure
 */
int blt_client_disconnect(struct blt_data_client_session_t *self)
{
	if (close(self->mySocket)) {
		self->status = BLT_DC_DISCONNECTED;
		return(1);
	}
	return(0);
}
